%huygens
% Program which applies Huygen's construction to generate the diffraction
% pattern for plane waves of wavelength lambda incident upon an aperture of
% width w wavelengths. Option to set N apertures of spacing s wavelengths.
%
% Waves are propagating upwards (i.e. aperture is horizontal).
%
% LAST UPDATED by Andy French 7th March 2019

function huygens(varargin)

%% INPUTS %%

%Line width
linewidth = 0.01;

%Points per semi-circular arc
P = 100;

%Spherical waves per aperture
A = 20;

%Number of wavelengths
W = 7;

%Number of apertures
N = 2;

%Aperture width in wavelengths
w = 7;

%Aperture wall thickness /wavelengths
t = 0.1;

%Spacing of aperture in wavelengths
s = 10;

%Set x and y limits
ylimits = [-t/2,W+1];
xlimits = [-W-0.5*N*s,0.5*N*s+W];

%Get inputs huygens function is called with arguments huygens(...)
if nargin~=0
    %Define aperture width, spacing, number, x and y limits
    w = varargin{1};
    s = varargin{2};
    N = varargin{3};
    xlimits = varargin{4};
    ylimits = varargin{5};
end

%%

%Create a figure and axes
fig = figure('color',[1 1 1],'name','huygens','renderer','opengl');
ax = axes('nextplot','add');

%Determine distance (in wavelengths) from start of first aperture to
%centre of 'grating'
x_centre = ( (N-1)*s + w )/2;

%Step through apertures and plot semicircles, for W wavelengths
for n=1:N
    %Determine y coordinates of start and end of aperture n
    xs = (n-1)*s - x_centre;
    xf = xs + w;
    
    %Generate y coordinates of semicircle centres
    xc = linspace(xs,xf,A);
    
    %Random line colour
    if N>1
        line_colour_RGB = rand(1,3);
    else
        line_colour_RGB = [1,0,0];
    end
    
    %Plot semicircles (1 to W wavelengths)
    for a=1:A
        for k=1:W
            [p,x,y] = semicircle( k, xc(a), 0, linewidth,line_colour_RGB , P );
        end
    end
end

%Fix axes scale
axis equal
axis manual
xlim( xlimits );
ylim( ylimits );
box on

%Plot boundary
patch( [xlimits,fliplr(xlimits)],[t/2,t/2,-t/2,-t/2],'k','edgecolor',[0,0,0] )

%Drill gaps for aperture in boundary
for n=1:N
    xs = (n-1)*s - x_centre ; xf = xs + w ;
    patch( [xs,xf,xf,xs],[t/2,t/2,-t/2,-t/2],'w','edgecolor',[1,1,1] );
end

%Set title
if N>1
    title(['huygens N=',num2str(N),' s=',num2str(s),'\lambda w=',num2str(w),'\lambda'],'fontsize',18)
else
    title(['huygens w=',num2str(w),'\lambda'],'fontsize',18)
end

%Save PNG file
print( fig, ['huygens N=',num2str(N),' s=',num2str(s),' w=',num2str(w),'.png'],...
    '-dpng','-r300' );
close(fig);

%%

%Plot a semicircle of radius r, centre coordinates (xc,yc), P points
function [p,x,y] = semicircle( r, xc, yc, linewidth, line_colour_RGB, P )
theta = linspace( 0,pi,P);
x = xc + r*cos(theta);
y = yc + r*sin(theta);
p = plot(x,y,'linewidth',linewidth,'color',line_colour_RGB);

%End of code